package com.mortals.xhx.webflux.service.impl;


import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.mortals.xhx.webflux.common.JsonUtils;
import com.mortals.xhx.webflux.enums.FlowChatSceneEnum;
import com.mortals.xhx.webflux.req.MessagesItem;
import com.mortals.xhx.webflux.req.RobotTransReq;
import com.mortals.xhx.webflux.template.AbstractFlowChatTemplate;
import com.mortals.xhx.webflux.factory.FLowChatRequest;
import com.mortals.xhx.webflux.factory.FlowChatContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service("baseChatService")
public class BaseChatService extends AbstractFlowChatTemplate {


    @Value("${chat.streamChatUrl:''}")
    private String streamChatUrl;

    @Value("${chat.auth-head:''}")
    private String authHead;

    @Value("${chat.auth-value:''}")
    private String authValue;


    private Integer num = 0;


    @Override
    public FlowChatSceneEnum getFlowChatEnum() {
        return FlowChatSceneEnum.BASE_CHAT;
    }


    @Override
    public String onNext(String data, FlowChatContext context) {

        if (StrUtil.equals("[DONE]", data)) {//[DONE]是消息结束标识
            return null;
        }
        // 解析响应数据项
        JSONObject bodyJson = JSONObject.parseObject(data);
        if (bodyJson == null) {
            return data;
        }

        /*if(ObjectUtils.isEmpty(context.getAnswer())){
            //创建一个初始化的answer
            JSONObject rspJson = new JSONObject();
            rspJson.put("id", context.getMsgId());
            rspJson.put("created", new Date().getTime());
            rspJson.put("model", "");

            JSONArray jsonArray = new JSONArray();
            JSONObject obj = new JSONObject();

            JSONObject delta = new JSONObject();
            delta.put("role", "assistant");
            delta.put("content", "");
            obj.put("delta", delta);

            obj.put("index", 0);
            obj.put("finish_reason", "null");

            jsonArray.add(obj);
            rspJson.put("choices", jsonArray);
            rspJson.put("object", "");

            context.getAnswer().append(".");

            context.getEmitter().next(rspJson.toJSONString());

        }
*/

       // context.getAnswer().append(answer);

        // String id = bodyJson.getString("id");

       /* if (num == 0) {
            JSONObject rspJson = new JSONObject();
            rspJson.put("id", context.getMsgId());
            rspJson.put("created", new Date().getTime());
            rspJson.put("model", "");

            JSONArray jsonArray = new JSONArray();
            JSONObject obj = new JSONObject();

            JSONObject delta = new JSONObject();
            delta.put("role", "assistant");
            delta.put("content", "");
            obj.put("delta", delta);

            obj.put("index", 0);
            obj.put("finish_reason", "null");

            jsonArray.add(obj);
            rspJson.put("choices", jsonArray);
            rspJson.put("object", "");
            num++;
            context.getEmitter().next(rspJson.toJSONString());
        }*/

        JSONArray choices = bodyJson.getJSONArray("choices");
        bodyJson.put("id", context.getMsgId());
        bodyJson.put("created", new Date().getTime());
        if (choices != null && choices.size() > 0) {
            JSONObject jsonObject = choices.getJSONObject(0);
            JSONObject delta = jsonObject.getJSONObject("delta");
            String role = delta.getString("role");

            delta.put("role", "");
            String content = delta.getString("content");
            if ("assistant".equals(role)) {
                context.getAnswer().append(content);
            }

            String finish_reason = jsonObject.getString("finish_reason");
            if ("stop".equals(finish_reason)) {
                delta.put("content", "");
                this.num = 0;
            }else{
                jsonObject.put("finish_reason", "null");
            }

        }

        return bodyJson.toJSONString();
    }

    @Override
    public void completed(FlowChatContext context) {
        // 更新对话
        updateChatMsg(context);
    }

    @Override
    public void error(FlowChatContext context) {
        // 更新对话
        updateChatMsg(context);
    }

    @Override
    protected FLowChatRequest buildRequest(FlowChatContext context) {
        //fastgptapi
        // 封装参数
        JSONObject params = new JSONObject();
        JSONArray jsonArray = new JSONArray();

        // 构建请求参数
        RobotTransReq robotTransReq = context.getRequestParam();
        //messageId
        String id = robotTransReq.getId();
        context.setMsgId(id);
        Integer maxTokens = robotTransReq.getMaxTokens();
        String enterpriseId = robotTransReq.getEnterpriseId();
        Boolean stream = robotTransReq.getStream();
        List<MessagesItem> messages = robotTransReq.getMessages();
        for (MessagesItem message : messages) {
            JSONObject obj = new JSONObject();
            obj.put("role", message.getRole());
            obj.put("content", message.getContent());
            jsonArray.add(obj);
            params.put("messages", jsonArray);
        }

        params.put("chatId", id);
        params.put("stream", stream);
        params.put("detail", false);
        params.put("responseChatItemId", "my_responseChatItemId");
        JSONObject variables = new JSONObject();
        variables.put("uid", "asdfadsfasfd2323");
        variables.put("name", "robot");
        params.put("variables", variables);

        return FLowChatRequest.builder().url(streamChatUrl).jsonBody(params.toJSONString()).build();
    }

    @Override
    protected HttpHeaders buildRequestHeaders(FlowChatContext context) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set(authHead, authValue);
        return headers;
    }

    @Override
    protected void doPreRequest(FlowChatContext context) {
        String query = "";
        // 保存query
        super.saveChatMsg(context, query);
    }
}
